// Code generated by go tool dist; DO NOT EDIT.
// This is a bootstrap copy of C:\bld\go_1546138896766\work\go\src\debug\dwarf\export_test.go

//line C:\bld\go_1546138896766\work\go\src\debug\dwarf\export_test.go:1
// Copyright 2017 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package dwarf

var PathJoin = pathJoin
